#ifndef RTConfig_h
#define RTConfig_h

//Singleton configuration class

class RTConfig {
	static RTConfig *_instance;

	int lightMode;
	int maxRefraction;
	int maxReflection;
	
	// numbers of rays to trace in a pixel
	int numPrimaryRays;

	bool texEnable;

	bool previewMode;

	float rayTMin;

	RTConfig() :	lightMode(1), maxRefraction(5), maxReflection(5), 
					texEnable(1), numPrimaryRays(1), rayTMin(0.0005f),
					testFlag(0), previewTexID(0), previewMode(0), 
					gpurtSafeMode(false), debugDisplay(false), 
					useFresnel(true), useGPURT(false) {
						
		drawTriangleMesh=true;
		drawTriangle=true;
		drawPlane=true;
		drawSphere=true;
		drawCubeMap=true;
		drawLight=true;
		previewSize=4.0f;

		drawHelpers=false;
	}
public:
	static RTConfig *getInstance(){
		if(!_instance)
			_instance=new RTConfig();

		return _instance;
	}


	enum RTCLightMode {RTC_LM_BLINN, RTC_LM_PHONG};
	enum RTCPreviewMode {RTC_PM_WINDOW, RTC_PM_CAMERA};

	float getRayTMin(){
		return rayTMin;
	}
	void setRayTMin(float tm){
		this->rayTMin=tm;
	}

	int getLightMode(){
		return lightMode;
	}
	void setLightMode(int lm){
		this->lightMode=lm;
	}

	int getPreviewMode(){
		return previewMode;
	}
	void setPreviewMode(int lm){
		this->previewMode=lm;
	}

	int getNumPrimaryRays(){
		return numPrimaryRays;
	}
	void setNumPrimaryRays(int n){
		if (n < 1)
			n = 1;
		this->numPrimaryRays=n;
	}
	
	bool getTexEnable(){
		return texEnable;
	}
	void setTexEnable(bool t){
		this->texEnable=t;
	}
	int getMaxRefraction(){
		return maxRefraction;
	}
	void setMaxRefraction(int mr){
		if (mr < 0)
			mr = 0;

		this->maxRefraction=mr;
	}
		
	int getMaxReflection(){
		return maxReflection;
	}
	void setMaxReflection(int mr){
		if (mr < 0)
			mr = 0;

		this->maxReflection=mr;
	}

	int previewTexID;
	bool testFlag;

	bool gpurtSafeMode;
	bool debugDisplay;
	

	bool useFresnel;
	bool useGPURT;

	bool drawTriangleMesh;
	bool drawTriangle;
	bool drawPlane;
	bool drawSphere;
	bool drawCubeMap;
	bool drawLight;

	bool drawHelpers;

	float previewSize;
};
#endif