#include "hit.h"

// Draw a point representing the intersection of the ray and the object
void Hit::draw() const{
	  // No intersection
	  if(time_>1.0E4)
		  return;

	  glPushAttrib(GL_ALL_ATTRIB_BITS);
	  glMatrixMode(GL_MODELVIEW);
	  glPushMatrix();

	  glDisable(GL_LIGHTING);
	  glPointSize(10.0f);
	  glDisable(GL_DEPTH_TEST);
	  //glDepthMask(false);


	  glBegin(GL_POINTS);
	  glColor4f(1.0,1.0,1.0,1.0);
	  glVertex3f(intersection_.x, intersection_.y, intersection_.z);
	  glEnd();

	  glPointSize(8.0f);
	  glBegin(GL_POINTS);
	  glColor4f(1.0,0.2,0.1,1.0);
	  glVertex3f(intersection_.x, intersection_.y, intersection_.z);
	  glEnd();

	  // Draw the normal of the hit
	  glColor4f(0,1,0,1);
	  glLineWidth(3.0f);
	  glBegin(GL_LINES);
	  glVertex3f(intersection_.x, intersection_.y, intersection_.z);
	  glVertex3f(intersection_.x+normal_.x/10.0f, intersection_.y+normal_.y/10.0f, intersection_.z+normal_.z/10.0f);
	  glEnd();
	 
/*
	  //TODO: draw the "cone" of the direction of the normal
	  float ax=acosf(normal_.y);
	  float ay=acosf(normal_.x);

	  if(normal_.z<0){
		  ax-=M_PI/2.0f;
		  ay-=M_PI/2.0f;

	  }
	  ax=M_PI/2.0f-ax;
	  ay=M_PI/2.0f-ay;

	  glTranslatef(intersection_.x, intersection_.y, intersection_.z);
	  glRotatef(ax*180/M_PI, 1,0,0);
	  glRotatef(ay*180/M_PI, 0,1,0);

	  glutSolidCone(0.1, 0.4, 10, 10 );
*/
	  glPopMatrix();
	  glPopAttrib();
  }


void Hit::setNormal(const qglviewer::Vec& n) { 
	normal_ = n;

	normalChanged=true;
}

qglviewer::Vec Hit::normal() const { 

	if(normalChanged){
		// Extension: Bump Mapping
		qglviewer::Vec bumpPerturb =  this->material().bumpMapVec(this->u(), this->v());

		if(bumpPerturb.norm()>0){
			qglviewer::Vec matL0(this->tangent_[0].x, this->tangent_[1].x, this->normal_.x);
			qglviewer::Vec matL1(this->tangent_[0].y, this->tangent_[1].y, this->normal_.y);
			qglviewer::Vec matL2(this->tangent_[0].z, this->tangent_[1].z, this->normal_.z);

			/*qglviewer::Vec matL0=this->tangent_[0];
			qglviewer::Vec matL1=this->tangent_[1];
			qglviewer::Vec matL2=this->normal_;*/

			qglviewer::Vec texSpaceNormal=bumpPerturb;

			//Changement de base du tangent space au monde
			normal_.x=matL0*texSpaceNormal;
			normal_.y=matL1*texSpaceNormal;
			normal_.z=matL2*texSpaceNormal;

			//normal_=bumpPerturb;

			normal_.normalize();

		}
		normalChanged=false;
	}

	return normal_;
}