#ifndef MATERIAL_H
#define MATERIAL_H

#include <QGLViewer/vec.h>
#include <qimage.h>
#include "color.h"

class Material
{
public :
  Material();

  enum TextureMode { MODULATE, BLEND, REPLACE };

  Color diffuseColor() const { return diffuseColor_; }
  Color diffuseColor(float u, float v) const;
  Color specularColor() const { return specularColor_; }
  Color reflectiveColor() const { return reflectiveColor_; }

  // "Extension: Refraction"
  Color refractiveColor() const { return refractiveColor_; }
  float refractiveIndex() const { return refractiveIndex_; }
 

  float specularCoefficient() const { return specularCoefficient_; }
  float textureScaleU() const { return textureScaleU_; }
  float textureScaleV() const { return textureScaleV_; }
  TextureMode textureMode() const { return textureMode_; }
  const QImage& texture() const { return texture_; }

  void setDiffuseColor(const Color& dc) { diffuseColor_ = dc; }
  void setSpecularColor(const Color& specularColor) { specularColor_ = specularColor; }
  void setReflectiveColor(const Color& reflectiveColor) { reflectiveColor_ = reflectiveColor; }

  // "Extension: Refraction"
  void setRefractiveColor(const Color& refractiveColor) { refractiveColor_ = refractiveColor; }
  void setRefractiveIndex(float refractiveIndex) { refractiveIndex_ = refractiveIndex; }


  void setSpecularCoefficient(float specularCoefficient) { specularCoefficient_ = specularCoefficient; }
  void setTexture(const QImage& texture) { texture_ = texture; }

  void setTextureScale(float textureScale) { setTextureScaleU(textureScale); setTextureScaleV(textureScale); }
  void setTextureScaleU(float textureScaleU) { if (fabs(textureScaleU) > 1e-3) textureScaleU_ = textureScaleU; }
  void setTextureScaleV(float textureScaleV) { if (fabs(textureScaleV) > 1e-3) textureScaleV_ = textureScaleV; }
  void setTextureMode(TextureMode textureMode) { textureMode_ = textureMode; }

  void loadTextureFromFile(QImage &qi, const QString& fileName);


  // "Extension: Perturbation de normales"
 // qglviewer::Vec bumpMapColor(float u, float v) const;
  qglviewer::Vec bumpMapVec(float u, float v) const;
  const QImage& bumpMapTexture() const { return bumpMapTexture_; }
  //void loadBumpMapTextureFromFile(const QString& fileName);

  void setBumpMapScale(float bumpMapScale) { setBumpMapScaleU(bumpMapScale); setBumpMapScaleV(bumpMapScale); }
  void setBumpMapScaleU(float bumpMapScaleU) { if (fabs(bumpMapScaleU) > 1e-3) bumpMapScaleU_ = bumpMapScaleU; }
  void setBumpMapScaleV(float bumpMapScaleV) { if (fabs(bumpMapScaleV) > 1e-3) bumpMapScaleV_ = bumpMapScaleV; }
  void setBumpMapHeight(float h) { bumpMapHeight_ = h; }

  float bumpMapScaleU() const { return bumpMapScaleU_; }
  float bumpMapScaleV() const { return bumpMapScaleV_; }
  float bumpMapHeight() const { return bumpMapHeight_; }


  ///Specular Map
  float specularMapColor(float u, float v) const;
  const QImage& specularMapTexture() const { return specularMapTexture_; }
 // void loadSpecularMapTextureFromFile(const QString& fileName);

  void setSpecularMapScale(float s) { setSpecularMapScaleU(s); setSpecularMapScaleV(s); }
  void setSpecularMapScaleU(float s) { if (fabs(s) > 1e-3) specularMapScaleU_ = s; }
  void setSpecularMapScaleV(float s) { if (fabs(s) > 1e-3) specularMapScaleV_ = s; }

  float specularMapScaleU() const { return specularMapScaleU_; }
  float specularMapScaleV() const { return specularMapScaleV_; }


  void initFromDOMElement(const QDomElement& e);
  
  //Image interpolated value
  Color imageInterpValue(const QImage &qi, float x, float y, float *alpha=NULL) const;

private:
  Color diffuseColor_, specularColor_, reflectiveColor_;
  float specularCoefficient_, textureScaleU_, textureScaleV_;
  TextureMode textureMode_;
  QImage texture_;

  // "Extension: refraction"
  Color refractiveColor_;
  float refractiveIndex_;

  // "Extension: Perturbation de normales"
  QImage bumpMapTexture_;
  float bumpMapHeight_, bumpMapScaleU_, bumpMapScaleV_;

  QImage specularMapTexture_;
  float specularMapHeight_, specularMapScaleU_, specularMapScaleV_;
};

#endif // MATERIAL_H
