#ifndef SCENE_H
#define SCENE_H

#include "object.h"
#include "light.h"
#include "camera.h"

#include <qlist.h>

//#include <qptrlist.h>

#include <QGLViewer/vec.h>


class Scene
{
public :
  Scene() : mini(0,0,0), maxi(0,0,0){
  }

  virtual ~Scene();

  void draw() const;
  void addObject(Object *o);
  void addLight(Light *l);
  void loadFromFile(const QString& filename);

  Object *getObject(int i){
	  return this->objectList.at(i);
  }
  int numObject(){
	  return this->objectList.count();
  }

  Light *getLight(int i){
	  i= i< 0 ? 0 : i%this->lightList.size();
	  return this->lightList.at(i);
  }
  int numLight(){
	  return this->lightList.count();
  }

  qglviewer::Vec center() const;
  float radius(const qglviewer::Vec& center) const;

  void setCamera(const Camera& camera) { camera_ = camera; }
  const Camera& camera() const { return camera_; }
  Camera& camera() { return camera_; }

  bool intersect(const Ray& ray, Hit& hit) const ;

  Color backgroundColor() const { return backgroundColor_;}

  Color illuminatedColor(const Hit& hit) const;
  Color illuminatedColorWithRefraction(const Hit& hit) const;


  int specialMode;
protected:
  void compMinMax();

private:
  Camera camera_;
  QList<Object *> objectList;
  QList<Light *> lightList;

  qglviewer::Vec mini, maxi;

  Color backgroundColor_;

};

#endif // SCENE_H
